@@ -70,27 +70,11 @@ bool al_init_image_addon(void)
          success |= al_register_bitmap_saver_f(".gif", _al_save_gdiplus_gif_f);
          success |= al_register_bitmap_saver_f(".png", _al_save_gdiplus_png_f);
          success |= al_register_bitmap_saver_f(".jpg", _al_save_gdiplus_jpg_f);
-         success |= al_register_bitmap_saver_f(".jpeg", _al_save_gdiplus_jpg_f);  
+         success |= al_register_bitmap_saver_f(".jpeg", _al_save_gdiplus_jpg_f);
       }
    }
 #endif
 
-#ifdef ALLEGRO_CFG_IIO_HAVE_ANDROID
-   {
-      char const *extensions[] = {".webp", ".jpg", ".jpeg", ".ico", ".gif", 
-         ".wbmp", ".png", NULL};
-      int i;
-
-
-      for (i = 0; extensions[i]; i++) {
-         success |= al_register_bitmap_loader(extensions[i], _al_load_android_bitmap);
-         success |= al_register_bitmap_loader_f(extensions[i], _al_load_android_bitmap_f);
-         //success |= al_register_bitmap_saver(extensions[i], _al_save_android_bitmap);
-      }
-      success |= al_register_bitmap_identifier(".webp", _al_identify_webp);
-   }
-#endif
-
 #ifdef ALLEGRO_CFG_IIO_HAVE_PNG
    success |= al_register_bitmap_loader(".png", _al_load_png);
    success |= al_register_bitmap_saver(".png", _al_save_png);
@@ -118,6 +102,41 @@ bool al_init_image_addon(void)
    success |= al_register_bitmap_identifier(".webp", _al_identify_webp);
 #endif
 
+#ifdef ALLEGRO_CFG_IIO_HAVE_FREEIMAGE
+   {
+      char const *extensions[] = {".ico", ".jng", ".koala", ".lbm",
+         ".iff", ".mng", ".pbm", ".pcd", ".pgm", ".ppm", ".ras", ".wbmp",
+         ".psd", ".cut", ".xpm", ".hdr", ".fax", ".sgi", ".exr", ".raw",
+         ".j2k", ".jp2", ".jpf", ".jpm", ".jpx", ".mj2", ".pfm", ".pict",
+         ".jxr", ".jbg", NULL};
+      int i;
+
+      if (_al_init_fi()) {
+         for (i = 0; extensions[i]; i++) {
+            success |= al_register_bitmap_loader(extensions[i], _al_load_fi_bitmap);
+            success |= al_register_bitmap_loader_f(extensions[i], _al_load_fi_bitmap_f);
+            success |= al_register_bitmap_identifier(extensions[i], _al_identify_fi);
+         }
+      }
+   }
+#endif
+
+#ifdef ALLEGRO_CFG_IIO_HAVE_ANDROID
+   {
+      char const *extensions[] = {".webp", ".jpg", ".jpeg", ".ico", ".gif",
+         ".wbmp", ".png", NULL};
+      int i;
+
+
+      for (i = 0; extensions[i]; i++) {
+         success |= al_register_bitmap_loader(extensions[i], _al_load_android_bitmap);
+         success |= al_register_bitmap_loader_f(extensions[i], _al_load_android_bitmap_f);
+         //success |= al_register_bitmap_saver(extensions[i], _al_save_android_bitmap);
+      }
+      success |= al_register_bitmap_identifier(".webp", _al_identify_webp);
+   }
+#endif
+
 #ifdef ALLEGRO_CFG_WANT_NATIVE_IMAGE_LOADER
 #ifdef ALLEGRO_IPHONE
    {
@@ -130,10 +149,6 @@ bool al_init_image_addon(void)
       }
    }
 #endif
-
-#ifdef ALLEGRO_MACOSX
-   success |= _al_osx_register_image_loader();
-#endif
 #endif
 
    if (success)
@@ -145,6 +160,14 @@ bool al_init_image_addon(void)
 }
 
 
+/* Function: al_is_image_addon_initialized
+ */
+bool al_is_image_addon_initialized(void)
+{
+   return iio_inited;
+}
+
+
 /* Function: al_shutdown_image_addon
  */
 void al_shutdown_image_addon(void)
