@@ -29,10 +29,9 @@ int main(int argc, char **argv)
    al_init_image_addon();
 
    al_set_new_bitmap_format(ALLEGRO_PIXEL_FORMAT_ARGB_8888);
-   al_set_new_bitmap_flags(ALLEGRO_MEMORY_BITMAP
-      | ALLEGRO_NO_PREMULTIPLIED_ALPHA);
+   al_set_new_bitmap_flags(ALLEGRO_MEMORY_BITMAP);
 
-   bitmap = al_load_bitmap(argv[1]);
+   bitmap = al_load_bitmap_flags(argv[1], ALLEGRO_NO_PREMULTIPLIED_ALPHA);
    if (!bitmap) {
       log_printf("Error loading input file\n");
       goto done;
