@@ -3,7 +3,7 @@
 
 #include "common.c"
 
-int main(void)
+int main(int argc, char **argv)
 {
    ALLEGRO_DISPLAY *display;
    ALLEGRO_BITMAP *icon1;
@@ -12,11 +12,15 @@ int main(void)
    ALLEGRO_TIMER *timer;
    int i;
 
+   (void)argc;
+   (void)argv;
+
    if (!al_init()) {
       abort_example("Could not init Allegro.\n");
    }
    al_install_keyboard();
    al_init_image_addon();
+   init_platform_specific();
 
    display = al_create_display(320, 200);
    if (!display) {
