@@ -44,7 +44,7 @@ static void initialize(void)
    if (!al_install_mouse()) {
       abort_example("Could not init mouse!\n");
    }
-   
+
    al_init_acodec_addon();
 
    if (!al_install_audio()) {
@@ -56,11 +56,11 @@ static void initialize(void)
 
    init_platform_specific();
 
-   display = al_create_display(640, 228);
+   display = al_create_display(640, 252);
    if (!display) {
       abort_example("Could not create display!\n");
    }
-   
+
    basic_font = al_load_font("data/font.tga", 0, 0);
    if (!basic_font) {
       abort_example("Could not load font!\n");
@@ -108,7 +108,7 @@ static void render(void)
    ALLEGRO_COLOR c = al_map_rgb(255, 255, 255);
 
    al_clear_to_color(al_map_rgb(64, 64, 128));
-   
+
    /* render "music player" */
    al_set_blender(ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_INVERSE_ALPHA);
    al_draw_textf(basic_font, c, 0, 0, 0, "Playing %s", stream_filename);
@@ -120,14 +120,15 @@ static void render(void)
    al_draw_line(10.0 + loop_end_pos, 46.0, 10.0 + loop_end_pos, 66.0, al_map_rgb(255, 0, 0), 0);
    al_draw_filled_rectangle(10.0 + slider_pos - 2.0, 48.0, 10.0 + slider_pos + 2.0, 64.0,
       al_map_rgb(224, 224, 224));
-   
+
    /* show help */
    al_draw_textf(basic_font, c, 0, 96, 0, "Drag the slider to seek.");
    al_draw_textf(basic_font, c, 0, 120, 0, "Middle-click to set loop start.");
    al_draw_textf(basic_font, c, 0, 144, 0, "Right-click to set loop end.");
    al_draw_textf(basic_font, c, 0, 168, 0, "Left/right arrows to seek.");
    al_draw_textf(basic_font, c, 0, 192, 0, "Space to pause.");
-   
+   al_draw_textf(basic_font, c, 0, 216, 0, "R to rewind.");
+
    al_flip_display();
 }
 
@@ -154,12 +155,14 @@ static void maybe_fiddle_sliders(int mx, int my)
       al_seek_audio_stream_secs(music_stream, seek_pos);
    }
    else if (mouse_button[2]) {
-      loop_end = seek_pos;
-      al_set_audio_stream_loop_secs(music_stream, loop_start, loop_end);
+      if (al_set_audio_stream_loop_secs(music_stream, loop_start, seek_pos)) {
+         loop_end = seek_pos;
+      }
    }
    else if (mouse_button[3]) {
-      loop_start = seek_pos;
-      al_set_audio_stream_loop_secs(music_stream, loop_start, loop_end);
+      if (al_set_audio_stream_loop_secs(music_stream, seek_pos, loop_end)) {
+         loop_start = seek_pos;
+      }
    }
 }
 
@@ -188,6 +191,11 @@ static void event_handler(const ALLEGRO_EVENT * event)
             if (!al_seek_audio_stream_secs(music_stream, pos))
                log_printf("seek error!\n");
          }
+         else if (event->keyboard.keycode == ALLEGRO_KEY_R) {
+            if (!al_rewind_audio_stream(music_stream)) {
+               log_printf("rewind error!\n");
+            }
+         }
          else if (event->keyboard.keycode == ALLEGRO_KEY_SPACE) {
             bool playing;
             playing = al_get_audio_stream_playing(music_stream);
@@ -258,6 +266,9 @@ int main(int argc, char *argv[])
          else if (!strcmp(s, "once")) {
             playmode = ALLEGRO_PLAYMODE_ONCE;
          }
+         else if (!strcmp(s, "loop_once")) {
+            playmode = ALLEGRO_PLAYMODE_LOOP_ONCE;
+         }
       }
       al_destroy_config(config);
    }
