@@ -22,6 +22,7 @@
 #include "allegro5/internal/aintern_bitmap.h"
 #include "allegro5/internal/aintern_blend.h"
 #include "allegro5/internal/aintern_pixels.h"
+#include "allegro5/internal/aintern_primitives.h"
 #include "allegro5/internal/aintern_tri_soft.h"
 #include <math.h>
 
@@ -132,7 +133,7 @@ static void shader_grad_any_init(uintptr_t state, ALLEGRO_VERTEX* v1, ALLEGRO_VE
    state_grad_any_2d* s = (state_grad_any_2d*)state;
 
    s->solid.target = al_get_target_bitmap();
-   
+
    s->off_x = v1->x - 0.5f;
    s->off_y = v1->y + 0.5f;
 
@@ -222,6 +223,7 @@ typedef struct {
    float off_y;
 
    ALLEGRO_BITMAP* texture;
+   ALLEGRO_BITMAP_WRAP default_wrap;
    int w, h;
 } state_texture_solid_any_2d;
 
@@ -242,7 +244,7 @@ static void shader_texture_solid_any_init(uintptr_t state, ALLEGRO_VERTEX* v1, A
 
    s->w = al_get_bitmap_width(s->texture);
    s->h = al_get_bitmap_height(s->texture);
-   
+
    if (det_u == 0.0f) {
       s->du_dx = s->du_dy = s->u_const = 0.0f;
       s->dv_dx = s->dv_dy = s->v_const = 0.0f;
@@ -319,7 +321,7 @@ static void shader_texture_grad_any_init(uintptr_t state, ALLEGRO_VERTEX* v1, AL
    PLANE_DETS(v, v1->v, v2->v, v3->v)
 
    state_texture_grad_any_2d* s = (state_texture_grad_any_2d*)state;
-   
+
    s->solid.target = al_get_target_bitmap();
    s->solid.w = al_get_bitmap_width(s->solid.texture);
    s->solid.h = al_get_bitmap_height(s->solid.texture);
@@ -712,6 +714,16 @@ void _al_triangle_2d(ALLEGRO_BITMAP* texture, ALLEGRO_VERTEX* v1, ALLEGRO_VERTEX
    }
 
    if (texture) {
+      ALLEGRO_BITMAP_WRAP wrap_u, wrap_v;
+      _al_get_bitmap_wrap(texture, &wrap_u, &wrap_v);
+
+      /*
+      XXX: Why are we using repeat for regular bitmaps by default? For some
+      reason the scanline drawers were always designed to repeat even for
+      regular bitmaps. This does not match what we do with OpenGL/Direct3D.
+      */
+      bool repeat = (wrap_u == ALLEGRO_BITMAP_WRAP_DEFAULT || wrap_u == ALLEGRO_BITMAP_WRAP_REPEAT) &&
+         (wrap_v == ALLEGRO_BITMAP_WRAP_DEFAULT || wrap_v == ALLEGRO_BITMAP_WRAP_REPEAT);
       if (grad) {
          state_texture_grad_any_2d state;
          state.solid.texture = texture;
@@ -731,9 +743,17 @@ void _al_triangle_2d(ALLEGRO_BITMAP* texture, ALLEGRO_VERTEX* v1, ALLEGRO_VERTEX
          state.texture = texture;
          if (shade) {
             if (white) {
-               _al_draw_soft_triangle(v1, v2, v3, (uintptr_t)&state, shader_texture_solid_any_init, shader_texture_solid_any_first, shader_texture_solid_any_step, shader_texture_solid_any_draw_shade_white);
+               if (repeat) {
+                  _al_draw_soft_triangle(v1, v2, v3, (uintptr_t)&state, shader_texture_solid_any_init, shader_texture_solid_any_first, shader_texture_solid_any_step, shader_texture_solid_any_draw_shade_white_repeat);
+               } else {
+                  _al_draw_soft_triangle(v1, v2, v3, (uintptr_t)&state, shader_texture_solid_any_init, shader_texture_solid_any_first, shader_texture_solid_any_step, shader_texture_solid_any_draw_shade_white);
+               }
             } else {
-               _al_draw_soft_triangle(v1, v2, v3, (uintptr_t)&state, shader_texture_solid_any_init, shader_texture_solid_any_first, shader_texture_solid_any_step, shader_texture_solid_any_draw_shade);
+               if (repeat) {
+                  _al_draw_soft_triangle(v1, v2, v3, (uintptr_t)&state, shader_texture_solid_any_init, shader_texture_solid_any_first, shader_texture_solid_any_step, shader_texture_solid_any_draw_shade_repeat);
+               } else {
+                  _al_draw_soft_triangle(v1, v2, v3, (uintptr_t)&state, shader_texture_solid_any_init, shader_texture_solid_any_first, shader_texture_solid_any_step, shader_texture_solid_any_draw_shade);
+               }
             }
          } else {
             if (white) {
