@@ -21,7 +21,11 @@
 #include "allegro5/internal/aintern_system.h"
 
 #if defined ALLEGRO_WITH_XWINDOWS
+#ifndef ALLEGRO_RASPBERRYPI
 #include "allegro5/platform/aintxglx.h"
+#else
+#include "allegro5/internal/aintern_raspberrypi.h"
+#endif
 #endif
 
 
@@ -31,11 +35,13 @@
  */
 void _al_register_system_interfaces(void)
 {
-#ifdef ALLEGRO_WITH_XWINDOWS
    ALLEGRO_SYSTEM_INTERFACE **add;
-   /* This is the only system driver right now */
+#if defined ALLEGRO_WITH_XWINDOWS && !defined ALLEGRO_RASPBERRYPI
    add = _al_vector_alloc_back(&_al_system_interfaces);
    *add = _al_system_xglx_driver();
+#elif defined ALLEGRO_RASPBERRYPI
+   add = _al_vector_alloc_back(&_al_system_interfaces);
+   *add = _al_system_raspberrypi_driver();
 #endif
 }
 
