@@ -21,8 +21,6 @@
 #include "allegro5/internal/aintern_direct3d.h"
 #include "allegro5/internal/aintern_shader.h"
 
-#ifdef ALLEGRO_CFG_SHADER_HLSL
-
 #include <d3dx9.h>
 #include <stdio.h>
 
@@ -30,19 +28,7 @@
 
 ALLEGRO_DEBUG_CHANNEL("shader")
 
-static _AL_VECTOR shaders;
-
-typedef struct ALLEGRO_SHADER_HLSL_S ALLEGRO_SHADER_HLSL_S;
-
-struct ALLEGRO_SHADER_HLSL_S
-{
-   ALLEGRO_SHADER shader;
-   LPD3DXEFFECT hlsl_shader;
-   int shader_model;
-};
-
-static const char *null_source = "";
-
+/* These few functions are included even when HLSL as a whole is not compiled in, as we're using them unconditionally in the D3D backend. */
 static const char *technique_source_vertex_v2 =
    "technique TECH\n"
    "{\n"
@@ -53,6 +39,52 @@ static const char *technique_source_vertex_v2 =
    "   }\n"
    "}\n";
 
+
+HRESULT _al_create_default_primitives_hlsl_vertex_shader(LPDIRECT3DDEVICE9 device, LPD3DXEFFECT *shader)
+{
+   ALLEGRO_USTR *full_source = al_ustr_newf("%s\n%s",
+   _al_get_default_hlsl_vertex_shader(), technique_source_vertex_v2);
+   LPD3DXBUFFER errors;
+
+   HRESULT res = _al_imp_D3DXCreateEffect(
+      device,
+      al_cstr(full_source),
+      al_ustr_size(full_source),
+      NULL,
+      NULL,
+      D3DXSHADER_PACKMATRIX_ROWMAJOR,
+      NULL,
+      shader,
+      &errors
+   );
+
+   al_ustr_free(full_source);
+   return res;
+}
+
+
+bool _al_hlsl_set_projview_matrix(
+   LPD3DXEFFECT effect, const ALLEGRO_TRANSFORM *t)
+{
+   HRESULT result = effect->SetMatrix(ALLEGRO_SHADER_VAR_PROJVIEW_MATRIX,
+      (LPD3DXMATRIX)t->m);
+   return result == D3D_OK;
+}
+
+
+#ifdef ALLEGRO_CFG_SHADER_HLSL
+
+static _AL_VECTOR shaders;
+
+struct ALLEGRO_SHADER_HLSL_S
+{
+   ALLEGRO_SHADER shader;
+   LPD3DXEFFECT hlsl_shader;
+   int shader_model;
+};
+
+static const char *null_source = "";
+
 static const char *technique_source_pixel_v2 =
    "technique TECH\n"
    "{\n"
@@ -481,14 +513,6 @@ static bool hlsl_set_shader_bool(ALLEGRO_SHADER *shader,
    return result == D3D_OK;
 }
 
-bool _al_hlsl_set_projview_matrix(
-   LPD3DXEFFECT effect, const ALLEGRO_TRANSFORM *t)
-{
-   HRESULT result = effect->SetMatrix(ALLEGRO_SHADER_VAR_PROJVIEW_MATRIX,
-      (LPD3DXMATRIX)t->m);
-   return result == D3D_OK;
-}
-
 void _al_d3d_init_shaders(void)
 {
    _al_vector_init(&shaders, sizeof(ALLEGRO_SHADER *));
