@@ -1,6 +1,6 @@
 #ifndef XICON_H
 #define XICON_H
 
-extern char **x11_xpm;
+extern ALLEGRO_BITMAP *_al_xwin_initial_icon;
 
 #endif // XICON_H
