@@ -83,9 +83,9 @@ void _al_xwin_set_size_hints(ALLEGRO_DISPLAY *d, int x_off, int y_off)
    wmhints = XAllocWMHints();
    wmhints->input = True;
    wmhints->flags = InputHint;
-   
+
    ALLEGRO_PATH *exepath = al_get_standard_path(ALLEGRO_EXENAME_PATH);
-   
+
    /* Setup the class hints so we can get an icon (AfterStep)
     * We must use the executable filename here.
     */
@@ -154,8 +154,8 @@ void _al_xwin_set_fullscreen_window(ALLEGRO_DISPLAY *display, int value)
 
    xev.xclient.data.l[1] = X11_ATOM(_NET_WM_STATE_FULLSCREEN);
    xev.xclient.data.l[2] = 0;
-   xev.xclient.data.l[3] = 0;
-   xev.xclient.data.l[4] = 1;
+   xev.xclient.data.l[3] = 1;  /* 1 = normal application source */
+   xev.xclient.data.l[4] = 0;
 
    XSendEvent(
       x11,
@@ -213,8 +213,6 @@ void _al_xwin_set_frame(ALLEGRO_DISPLAY *display, bool frame_on)
    Display *x11 = system->x11display;
    Atom hints;
 
-   _al_mutex_lock(&system->lock);
-
 #if 1
    /* This code is taken from the GDK sources. So it works perfectly in Gnome,
     * no idea if it will work anywhere else. X11 documentation itself only
@@ -239,8 +237,6 @@ void _al_xwin_set_frame(ALLEGRO_DISPLAY *display, bool frame_on)
          display->flags |= ALLEGRO_FRAMELESS;
    }
 #endif
-
-   _al_mutex_unlock(&system->lock);
 }
 
 
@@ -324,6 +320,9 @@ void _al_xwin_set_icons(ALLEGRO_DISPLAY *d,
 }
 
 
+/* Note: The system mutex must be locked (exactly once) before
+ * calling this as we call _al_display_xglx_await_resize.
+ */
 void _al_xwin_maximize(ALLEGRO_DISPLAY *display, bool maximized)
 {
 #ifndef ALLEGRO_RASPBERRYPI
@@ -400,4 +399,38 @@ XID al_get_x_window_id(ALLEGRO_DISPLAY *display)
    return ((ALLEGRO_DISPLAY_XGLX*)display)->window;
 }
 
+
+// Note: this only seems to work after the window has been mapped
+void _al_xwin_get_borders(ALLEGRO_DISPLAY *display) {
+   ALLEGRO_DISPLAY_XGLX *glx = (ALLEGRO_DISPLAY_XGLX *)display;
+   ALLEGRO_SYSTEM_XGLX *system = (void *)al_get_system_driver();
+   Display *x11 = system->x11display;
+
+   Atom type;
+   int format;
+   unsigned long nitems, bytes_after;
+   unsigned char *property;
+   Atom frame_extents = X11_ATOM(_NET_FRAME_EXTENTS);
+   if (XGetWindowProperty(x11, glx->window,
+         frame_extents, 0, 16, 0, XA_CARDINAL,
+         &type, &format, &nitems, &bytes_after, &property) == Success) {
+      if (type != None && nitems == 4) {
+         glx->border_left = (int)((long *)property)[0];
+         glx->border_right = (int)((long *)property)[1];
+         glx->border_top = (int)((long *)property)[2];
+         glx->border_bottom = (int)((long *)property)[3];
+         glx->borders_known = true;
+         ALLEGRO_DEBUG("_NET_FRAME_EXTENTS: %d %d %d %d\n", glx->border_left, glx->border_top,
+            glx->border_right, glx->border_bottom);
+      }
+      else {
+         ALLEGRO_DEBUG("Unexpected _NET_FRAME_EXTENTS format: nitems=%lu\n", nitems);
+      }
+      XFree(property);
+   }
+   else {
+      ALLEGRO_DEBUG("Could not read _NET_FRAME_EXTENTS\n");
+   }
+}
+
 /* vim: set sts=3 sw=3 et: */
